/***************************************************************************
    utils.c  -  description
    -----------------------
    begin                : pi lut 11 09:59:54 CET 2000
    copyright            : (C) 2000 by Tomasz Fiszer
    email                : profis@kki.net.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <signal.h>
#include <syslog.h>
#include <fcntl.h>
#include <time.h>
#include "defs.h"
#include "struct.h"
#include "utils.h"

static char szInternalCmd[MAX_STRING] = "";

void TimeoutCommand();

//
// Printing errors to STDOUT
//
void PrintError(char *szErrorString, int ierr, int itype)
{
    MsgLogError(szErrorString);
    printf("\n");
    if (ierr && itype == 0)
	printf("%s : %d\n", szErrorString, ierr);
    else
	if (ierr && itype == 1)
	    printf("%s : %s\n", szErrorString, strerror(ierr));
	else
	    if (!ierr)
		printf("%s\n", szErrorString);
    printf("\nPlease examine log file <%s>, or syslog \"powersoft\"\n", LOG_FILE);
}

//
// Check for Root privileges
//
int CheckSuperUser()
{
    if ((getuid() != 0) && (geteuid() != 0))
    {
	PrintError(STRING_ERROR_NOT_ROOT, 0, 0);
	return 0;
    }
    else
	return 1;
}

//
// Create file
//
int CreateFile(const char *szPath)
{
    int		fd;
    fd = open(szPath, O_CREAT | O_WRONLY, 0644);
    if (fd >= 0)
	close(fd);
    return fd;
}

//
// Create file
//
FILE *OpenFile(const char *szPath)
{
    FILE	*fd;
    fd = fopen(szPath, "wr");
    return fd;
}

//
// Delete file
//
int DeleteFile(const char *szPath)
{
    unlink(szPath);
    return *szPath;
}

//
// Makeing Process identifier
//
void CreatePID()
{
    pid_t	pPid;
    FILE	*fPid;
    
    unlink(POWERSOFTPID);
    fPid = fopen(POWERSOFTPID, "w");
    if (fPid != NULL) 
    {
	pPid = getpid();
    	fprintf(fPid, "%d\n", pPid);
    	fclose(fPid);
    }
}

//
// Opens system log for PowerSoft
//
void OpenLog()
{
    openlog("powersoft", LOG_PID | LOG_CONS, LOG_DAEMON);
}

//
// Close system log for PowerSoft
//
void CloseLog()
{
    closelog();
}

//
// Write a message to system log
//
void MsgLog(char *szMsg)
{
    syslog(LOG_NOTICE, szMsg);
}

//
// Write a error message to system log
//
void MsgLogError(char *szMsg)
{
    syslog(LOG_ERR, szMsg);
}

// Printing To log file
int PrintToLog(char *format, ...)
{
    va_list	ap;
    char	szTime[32], szMsgBuffer[256];
    time_t	tNow;
    int		iRet;
    FILE	*fl;
    
    fl = fopen(LOG_FILE, "a+");
    if (fl == NULL)
	return FAILURE;
    time(&tNow);
    strftime(szTime, sizeof(szTime), "%b %d %X", localtime(&tNow));
    fprintf(fl, "POWERSOFT: [%s] ", szTime);

    va_start(ap, format);
    iRet = vfprintf(fl, format, ap);
    vsprintf(szMsgBuffer, format, ap);
    va_end(ap);

    MsgLog(szMsgBuffer);
    if (fflush(fl))
	iRet = -1;
    fclose(fl);
    return(iRet);
}

// Printing To log file
int PrintToLogNet(char *format, ...)
{
    va_list	ap;
    char	szTime[32];
    time_t	tNow;
    int		iRet;
    FILE	*fl;
    
    fl = fopen(NET_LOG_FILE, "a+");
    if (fl == NULL)
	return FAILURE;
    va_start(ap, format);

    time(&tNow);
    strftime(szTime, sizeof(szTime), "%b %d %X", localtime(&tNow));
    fprintf(fl, "POWERSOFT: [%s] ", szTime);

    iRet = vfprintf(fl, format, ap);

    if (fflush(fl))
	iRet = -1;
    fclose(fl);
    va_end(ap);
    return(iRet);
}

//
// Alarm Timer
//
void InitAlarmTimer(int iTime, void (*CallbackHandler)(int))
{
    signal(SIGALRM, CallbackHandler);
    alarm(iTime);
}

//
// Initialization of thread signals
//
void InitThreadSignals() 
{
    signal(SIGHUP, TerminateThread);
    signal(SIGINT, TerminateThread);
    signal(SIGTERM, TerminateThread);
    signal(SIGCHLD, SIG_DFL);
    signal(SIGKILL, TerminateThread);
}

//
// Initialization of system signals
//
void InitSystemSignals() 
{
    signal(SIGHUP, Terminate);
    signal(SIGINT, Terminate);
    signal(SIGTERM, Terminate);
    signal(SIGCHLD, Terminate);
    signal(SIGKILL, Terminate);
}

//
// Restoring system signals
//
void RestoreSystemSignals() 
{
    signal(SIGALRM, SIG_DFL);
    signal(SIGHUP, SIG_DFL);
    signal(SIGINT, SIG_DFL);
    signal(SIGTERM, SIG_DFL);
    signal(SIGCHLD, SIG_DFL);
    signal(SIGKILL, SIG_DFL);
}

//
// Wait for termination of child process
//
void StandbyForTermination()
{
    int iWait;
    wait(&iWait);
}

//
// Loop
//
void Standby(int iSleep)
{
    if (iSleep < 0)
	sleep(-iSleep);
    else
	if (iSleep > 0)
	    sleep(iSleep);
	else
	    for (;;)
		sleep(1000);
}

int IsStatusSet(unsigned long dwSW, unsigned long dwBits)
{
    if ((dwSW & dwBits) == dwBits) 
	return 1;
    return 0;
}

unsigned long SetStatusWord(unsigned long *dwSW, unsigned long dwBits)
{
    *dwSW &= ~(dwBits);
    *dwSW |= dwBits;
    return (*dwSW);
}

unsigned long ClearStatusWord(unsigned long *dwSW, unsigned long dwBits)
{
    *dwSW &= ~(dwBits);
    return (*dwSW);
}

void BroadcastMessage(UPS_INFO *Ups, char *szMsg)
{
    char szOutMsg[MAX_STRING*2];
    
    if (Ups->uiMsgEnabled)
    {
	//
	// Using simple message wall procedure 
	// "wall" without parameter defined as file is treated as message
	//
	sprintf(szOutMsg, "wall %s", szMsg);
	system(szOutMsg);
	fflush(stdout);
    }
}

void BroadcastMessageInfo(UPS_INFO *Ups, char *szMsg)
{
    char	szTime[MAX_STRING*2] = "",
		szBuff[MAX_STRING*2] = "";

    if (strstr(szMsg, "%%m") != 0)
    {
	sprintf(szTime, "%s%ld", strncpy(szBuff, szMsg, strcspn(szMsg, "%%")), 
	    Ups->ulTimeToShutdown1);
	strcat(szTime, strchr(szMsg, '%') + 3);
    }else
	strcpy(szTime, szMsg);
    strcat(szTime, "\n");
    if (Ups->uiMsgEnabled)
    {
	sprintf(szBuff, "wall <<EOF\n%s\nEOF", szTime);
	system(szBuff);
	fflush(stdout);
    }
}

void RunCommand(UPS_INFO *Ups, char *szCmd)
{
    if (Ups->uiCmdEnabled)
    {
	system(szCmd);
	fflush(stdout);
    }
}

void RunTimeoutCommand(UPS_INFO *Ups, char *szCmd, int iTimeout)
{
    if ((Ups->uiCmdEnabled != 1) && (strlen(szInternalCmd) == 0))
    {
	strcpy(szInternalCmd, szCmd);
	signal(SIGALRM, TimeoutCommand);
	alarm(iTimeout);
    }
}

void TimeoutCommand()
{
    if (strlen(szInternalCmd))
    {
	system(szInternalCmd);
	fflush(stdout);
	*szInternalCmd = 0;
    }
}

//
// End of file
//
